# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"read_file": "Importing.ipynb",
         "extract_rawfile_unique_values": "Importing.ipynb",
         "import_spectronaut_data": "Importing.ipynb",
         "import_maxquant_data": "Importing.ipynb",
         "convert_ap_mq_mod": "Importing.ipynb",
         "import_alphapept_data": "Importing.ipynb",
         "convert_diann_mq_mod": "Importing.ipynb",
         "import_diann_data": "Importing.ipynb",
         "convert_fragpipe_mq_mod": "Importing.ipynb",
         "import_fragpipe_data": "Importing.ipynb",
         "import_data": "Importing.ipynb",
         "extract_uniprot_id": "Preprocessing.ipynb",
         "expand_protein_ids": "Preprocessing.ipynb",
         "pep_position_helper": "Preprocessing.ipynb",
         "get_peptide_position": "Preprocessing.ipynb",
         "get_ptm_sites": "Preprocessing.ipynb",
         "get_modifications": "Preprocessing.ipynb",
         "format_input_data": "Preprocessing.ipynb",
         "format_uniprot_annotation": "SequencePlot.ipynb",
         "ptm_shape_dict": "SequencePlot.ipynb",
         "get_plot_data": "SequencePlot.ipynb",
         "plot_single_peptide_traces": "SequencePlot.ipynb",
         "custom_color_palettes": "SequencePlot.ipynb",
         "uniprot_color_dict": "SequencePlot.ipynb",
         "aa_color_dict": "SequencePlot.ipynb",
         "get_quality_category": "SequencePlot.ipynb",
         "get_exposure_category": "SequencePlot.ipynb",
         "get_alphafold_annotation": "SequencePlot.ipynb",
         "plot_peptide_traces": "SequencePlot.ipynb",
         "extract_annotation": "SequencePlot.ipynb",
         "manipulate_cif": "SequencePlot.ipynb",
         "adjust_html": "SequencePlot.ipynb",
         "get_ms_concensus": "SequencePlot.ipynb",
         "plot_3d_structure": "SequencePlot.ipynb",
         "create_pdf_report": "SequencePlot.ipynb",
         "extract_note": "Uniprot_integration.ipynb",
         "extract_note_end": "Uniprot_integration.ipynb",
         "resolve_unclear_position": "Uniprot_integration.ipynb",
         "extract_positions": "Uniprot_integration.ipynb",
         "preprocess_uniprot": "Uniprot_integration.ipynb",
         "uniprot_feature_dict": "Uniprot_integration.ipynb",
         "all_organisms": "organisms_data.ipynb",
         "import_fasta": "organisms_data.ipynb",
         "import_uniprot_annotation": "organisms_data.ipynb",
         "protease_dict": "proteolytic_cleavage.ipynb",
         "get_cleavage_sites": "proteolytic_cleavage.ipynb"}

modules = ["importing.py",
           "preprocessing.py",
           "sequenceplot.py",
           "uniprot_integration.py",
           "organisms_data.py",
           "proteolytic_cleavage.py"]

doc_url = "https://mannlabs.github.io/alphamap/"

git_url = "https://github.com/mannlabs/alphamap/tree/master/"

def custom_doc_links(name): return None
